@@ -1,6 +1,7 @@
 """Define configuration objects.
 """
 import collections
+import datetime
 import errno
 import mimetypes
 import os
@@ -15,7 +16,7 @@ from aspen.hooks import Hooks
 from aspen.configuration import parse
 from aspen.configuration.exceptions import ConfigurationError
 from aspen.configuration.options import OptionParser, DEFAULT
-from aspen.utils import ascii_dammit
+from aspen.utils import ascii_dammit, to_rfc822, utcnow
 
 
 # Nicer defaultdict
@@ -27,11 +28,13 @@ NO_DEFAULT = object()
 class NicerDefaultDict(collections.defaultdict):
     """Subclass to support .default assignment.
     """
-    
+
     __default = ''
     def _get_default(self, name):
+        """property getter for default property"""
         return self.__default
     def _set_default(self, value):
+        """property setter for default property"""
         self.default_factory = lambda: value
         self.__default = value
     default = property(_get_default, _set_default)
@@ -39,14 +42,14 @@ class NicerDefaultDict(collections.defaultdict):
 
     def get(self, name, default=NO_DEFAULT):
         if default is NO_DEFAULT:
-            default = self.default 
+            default = self.default
         collections.defaultdict.get(self, name, default)
 
 
 # Defaults
 # ========
-# The from_unicode callable converts from unicode to whatever format is 
-# required by the variable, raising ValueError appropriately. Note that 
+# The from_unicode callable converts from unicode to whatever format is
+# required by the variable, raising ValueError appropriately. Note that
 # name is supposed to match the options in our optparser. I like it wet.
 
 KNOBS = \
@@ -66,7 +69,7 @@ KNOBS = \
     , 'charset_dynamic':    (u'UTF-8', parse.charset)
     , 'charset_static':     (None, parse.charset)
     , 'indices':            ( lambda: [u'index.html', u'index.json', u'index']
-                            , parse.list_ 
+                            , parse.list_
                              )
     , 'list_directories':   (False, parse.yes_no)
     , 'media_type_default': ('text/plain', parse.media_type)
@@ -84,8 +87,13 @@ class Configurable(object):
     """Mixin object for aggregating configuration from several sources.
     """
 
+    protected = False  # Set to True to require authentication for all
+                       # requests.
+
     @classmethod
     def from_argv(cls, argv):
+        """return a Configurable based on the passed-in arguments list
+        """
         configurable = cls()
         configurable.configure(argv)
         return configurable
@@ -100,7 +108,7 @@ class Configurable(object):
         if name_in_context:
             assert isinstance(flat, unicode) # sanity check
             name_in_context = " %s=%s" % (name_in_context, flat)
-        out = "  %-22s %-30s %-24s" 
+        out = "  %-22s %-30s %-24s"
         return out % (name, hydrated, context + name_in_context)
 
     def set(self, name, raw, from_unicode, context, name_in_context):
@@ -197,6 +205,24 @@ class Configurable(object):
         # Set some attributes.
         # ====================
 
+
+        def safe_getcwd(errorstr):
+            try:
+                # If the working directory no longer exists, then the following
+                # will raise OSError: [Errno 2] No such file or directory. I
+                # swear I've seen this under supervisor, though I don't have
+                # steps to reproduce. :-(  To get around this you specify a
+                # www_root explicitly, or you can use supervisor's cwd
+                # facility.
+
+                return os.getcwd()
+            except OSError, err:
+                if err.errno != errno.ENOENT:
+                    raise
+                raise ConfigurationError(errorstr)
+
+
+
         # LOGGING_THRESHOLD
         # -----------------
         # This is initially set to -1 and not 0 so that we can tell if the user
@@ -206,31 +232,8 @@ class Configurable(object):
             aspen.logging.LOGGING_THRESHOLD = self.logging_threshold
         # Now that we know the user's desires, we can log appropriately.
         aspen.log_dammit(os.linesep.join(msgs))
-       
-        # www_root
-        if self.www_root is None:
-            try:
 
-                # If the working directory no longer exists, then the following
-                # will raise OSError: [Errno 2] No such file or directory. I
-                # swear I've seen this under supervisor, though I don't have
-                # steps to reproduce. :-(  To get around this you specify a
-                # www_root explicitly, or you can use supervisor's cwd
-                # facility.
-
-                self.www_root = os.getcwd()
-            except OSError, err:
-                if err.errno != errno.ENOENT:
-                    raise
-                raise ConfigurationError("Could not get a current working "
-                                         "directory. You can specify "
-                                         "ASPEN_WWW_ROOT in the environment, "
-                                         "or --www_root on the command line.")
-
-        self.www_root = os.path.realpath(self.www_root)
-        os.chdir(self.www_root)
-
-        # project root 
+        # project root
         if self.project_root is None:
             aspen.log_dammit("project_root not configured (no template bases, "
                              "etc.).")
@@ -238,11 +241,14 @@ class Configurable(object):
         else:
             # canonicalize it
             if not os.path.isabs(self.project_root):
-                aspen.log_dammit("project_root is relative: '%s'." 
+                aspen.log_dammit("project_root is relative: '%s'."
                                  % self.project_root)
-                self.project_root = os.path.join( self.www_root
-                                                , self.project_root
-                                                 )
+                cwd = safe_getcwd("Could not get a current working "
+                                  "directory. You can specify "
+                                  "ASPEN_PROJECT_ROOT in the environment, "
+                                  "or --project_root on the command line.")
+                self.project_root = os.path.join(cwd, self.project_root)
+
             self.project_root = os.path.realpath(self.project_root)
             aspen.log_dammit("project_root set to %s." % self.project_root)
 
@@ -259,13 +265,23 @@ class Configurable(object):
             # PYTHONPATH
             sys.path.insert(0, self.project_root)
 
+        # www_root
+        if self.www_root is None:
+            self.www_root = safe_getcwd("Could not get a current working "
+                                         "directory. You can specify "
+                                         "ASPEN_WWW_ROOT in the environment, "
+                                         "or --www_root on the command line.")
+
+        self.www_root = os.path.realpath(self.www_root)
+        os.chdir(self.www_root)
+
         # renderers
         self.renderer_factories = {}
         for name in aspen.RENDERERS:
             # Pre-populate renderers so we can report on ImportErrors early
             try:
                 capture = {}
-                python_syntax = 'from aspen.renderers.%s_ import Factory' 
+                python_syntax = 'from aspen.renderers.%s import Factory'
                 exec python_syntax % name in capture
                 make_renderer = capture['Factory'](self)
             except ImportError, err:
@@ -274,7 +290,7 @@ class Configurable(object):
 
         default_renderer = self.renderer_factories[self.renderer_default]
         if isinstance(default_renderer, ImportError):
-            msg = "\033[1;31mImportError loading the default renderer, %s:\033[0m" 
+            msg = "\033[1;31mImportError loading the default renderer, %s:\033[0m"
             aspen.log_dammit(msg % self.renderer_default)
             raise default_renderer
 
@@ -299,17 +315,17 @@ class Configurable(object):
         mimetypes.init()
 
         # network_engine
-        try: 
+        try:
             capture = {}
-            python_syntax = 'from aspen.network_engines.%s_ import Engine' 
+            python_syntax = 'from aspen.network_engines.%s_ import Engine'
             exec python_syntax % self.network_engine in capture
             Engine = capture['Engine']
         except ImportError:
-            # ANSI colors: 
+            # ANSI colors:
             #   http://stackoverflow.com/questions/287871/
             #   http://en.wikipedia.org/wiki/ANSI_escape_code#CSI_codes
             #   XXX consider http://pypi.python.org/pypi/colorama
-            msg = "\033[1;31mImportError loading the %s network engine:\033[0m" 
+            msg = "\033[1;31mImportError loading the %s network engine:\033[0m"
             aspen.log_dammit(msg % self.network_engine)
             raise
         self.network_engine = Engine(self.network_engine, self)
@@ -334,7 +350,7 @@ class Configurable(object):
         # Finally, exec any configuration scripts.
         # ========================================
         # The user gets self as 'website' inside their configuration scripts.
-       
+
         for filepath in self.configuration_scripts:
             if not filepath.startswith(os.sep):
                 if self.project_root is None:
@@ -346,8 +362,8 @@ class Configurable(object):
             try:
                 execfile(filepath, {'website': self})
             except IOError, err:
-                # I was checking os.path.isfile for these, but then we have a 
-                # race condition that smells to me like a potential security 
+                # I was checking os.path.isfile for these, but then we have a
+                # race condition that smells to me like a potential security
                 # vulnerability.
                 if err.errno == errno.ENOENT:
                     msg = ("The configuration script %s doesn't seem to "
